module deriv

  implicit none

contains

! Calcola la derivata seconda nello spazio spettrale per uno sviluppo di
! Chebyshev.
! ft = Vettore contenente i coefficienti di Chebyshev (input)
! dxx = Vettore contenente i coefficienti della derivata seconda (output)
! nt = Ordine della trasformata di Chebyshev (input)
! Nota: dxx puo' essere = ft

  subroutine derxx( ft, dxx, Nt )

    double precision, intent(IN) :: ft(:)
    double precision, intent(OUT) :: dxx(:)
    integer, intent(IN) :: Nt
    integer :: is, ic, l
    double precision :: a, b, c

    is = Nt + 2
    a = 4 * Nt * ( Nt - 1 ) * ft( is - 1 )
    b = 4 * ( Nt - 1 ) * ( Nt - 2 ) * ft( is - 2 )
    dxx( is ) = 0.d0
    dxx( is - 1 ) = 0.d0
    dxx( is - 2 ) = 0.d0
    do l = 1, Nt-3
       ic = Nt-l-3
       c = 4 * ( ic + 2 ) * ( ic + 1 ) * ( ft( is - l - 2 ) + &
            a / ( ic * ic + 4 * ic + 3 ) / 2.d0 - dxx( is - l ) / &
            ( ic + 2 ) / ( ic + 3 ) / 4.d0 )
       dxx( is - l - 2 ) = a
       a = b
       b = c
    enddo
    dxx( is - 1 ) = a
    dxx( is - 2 ) = b / 2.d0

    return

  end subroutine derxx


! Calcola la derivata prima nello spazio spettrale per uno sviluppo di
! Chebyshev.
! ft = Vettore contenente i coefficienti di Chebyshev (input)
! dx = Vettore contenente i coefficienti della derivata prima (output)
! nt = Ordine della trasformata di Chebyshev (input)
! Nota: dxx puo' essere = ft

   subroutine derx( ft, dx, Nt )

      double precision, intent(IN) :: ft(:)
      double precision, intent(OUT) :: dx(:)
      integer, intent(IN) :: Nt
      integer :: is, ip, isl, l
      double precision :: pass(2)

      is = Nt+2
      ip = 1
      dx( is ) = 0.d0
      pass( 1 ) = 0.d0
      do l = 1, Nt
         isl = is - l
         pass( ( ip + 3 ) / 2 ) = 2 * ft( isl ) * ( Nt - l + 1 ) +dx( isl + 1 )
         dx( isl ) = pass( ( -ip + 3 ) / 2 )
         ip = -ip
      enddo
      dx( is - Nt - 1 ) = pass( ( -ip + 3 ) / 2 ) / 2.d0

      return

   end subroutine derx


! Calcola l'integrale di una funzione a partire dallo sviluppo di 
! Chebyshev nello spazio spettrale.
! f = funzione di cui calcolare l'integrale. Dimensione f(Nt+2) (input)
! fint = integrale della funzione (output)
! Nt = ordine della trasformata di Chebyshev

   subroutine intg( f, fint, Nt )

      double precision, intent(IN) :: f(:)
      double precision, intent(OUT) :: fint
      integer, intent(IN) :: Nt
      integer :: k
      double precision :: pi, sum

      pi = dacos( -1.d0 )
      sum = 0.d0
      do k = 1, Nt + 1, 2
         sum = sum + f( k ) * ( -2.d0 ) / ( ( k - 1.d0 ) * ( k - 1.d0 ) - 1.d0 )
      enddo
      fint = 2.d0 * pi * sum

      return

   end subroutine intg

end module deriv
