program parabolica

  use chebfft
  use deriv

  implicit none

  integer :: Nt, Ntp1, Ntp2, Nend, Nprint
  integer :: i, istep
  double precision :: dt, nu, Tend, Tprint, tt, kappa
  double precision :: sigma, sigma2, csign, alfa, beta
  double precision, allocatable, dimension (:) :: uf, us, d1uf, d1us, d2uf, d2us, &
       xch, alp, alm, wch
  double precision, allocatable, dimension (:,:) :: chfns

! Lettura dei dati da file 'input.dat'

  open (file='input.dat',unit=15,status='UNKNOWN')
  read(15,*) Nt
  read(15,*) dt
  read(15,*) nu
  read(15,*) Tend
  read(15,*) Tprint
  close (unit=15)

! Dimensionamento vettori

  Ntp1 = Nt + 1
  Ntp2 = Nt + 2
  allocate ( uf( Ntp1 ), d1uf( Ntp1 ), d2uf( Ntp1 ), us( Ntp2 ), d1us( Ntp2 ), d2us( Ntp2 ) )
  allocate ( xch( Ntp1 ), chfns( Ntp1, 4 ), alp( Ntp1 ), alm( Ntp1 ), wch( Ntp2 ) )

! Inizializzazione variabili

  tt = 0.d0
  Nend = nint( Tend / dt )
  Nprint = nint( Tprint / dt )

! Inizializzazione delle trasformate di Chebyshev e Fourier

  call initch( xch, chfns, Nt)

! Definizione delle condizioni iniziali

!  sigma = 0.1d0
!  sigma2 = sigma * sigma
!  kappa = 1.d0 / sigma2 * dexp( - 1.d0 / ( 2.d0 * sigma2 ) )
!  do i = 1, Nt/2
!     uf( i ) = dexp( - ( xch(i) * xch( i ) ) / ( 2.d0 * sigma2 ) ) - kappa * xch( i )
!  enddo
!  uf( Nt/2 + 1 ) = 1.d0
!  do i = Nt/2+2, Ntp1
!     uf( i ) = dexp( - ( xch(i) * xch( i ) ) / ( 2.d0 * sigma2 ) ) + kappa * xch( i )
!  enddo

!  sigma = 0.1d0
!  do i = 1, Ntp1
!     uf( i ) = dexp( - ( ( xch(i) - xch(1) ) ** 2.d0 ) / ( 2.d0 * sigma ) )
!  enddo

   do i = 1, Ntp1
      uf( i ) = 0.d0
   enddo
   uf( Nt/2 + 1 ) = 1.d0

! Trasformata della condizione iniziale

  call cheb( uf, us, wch, chfns, Nt, +1 )

! Condizioni al bordo

  csign = 1.d0
  do i = 1, Ntp1
! Funzione nulla sul bordo (destro e sinistro) -> Dirichlet
!     alp( i ) = 1.d0
!     alm( i ) = csign
! Derivata nulla sul bordo (destro e sinistro) -> Neumann
     alp( i ) = ( i - 1.d0 ) * ( i - 1.d0 )
     alm( i ) = -csign * ( i - 1.d0 ) * (i - 1.d0 )
     csign = -csign
  enddo

! Verifica condizioni al bordo

  call derx( us, d1us, Nt )
  call cheb( d1uf, d1us, wch, chfns, Nt, -1 )
do i=1,Ntp1
write(16,*) xch(i),d1uf(i)
enddo
  open ( file='bc.dat', unit=15, status = 'UNKNOWN' )
  write( 15, * ) d1uf( 1 ), d1uf( Ntp1 )
  close( unit = 15 )

! Output della condizione iniziale

  call output( xch, uf, tt, Ntp1)
  open ( file='integrale.dat', unit=16, status = 'UNKNOWN' )
  write(16,*) tt, trapezi( uf, xch, Ntp1 )
  close( unit = 16 )

! Inizio ciclo temporale

  do istep = 1, Nend

! Calcolo secondi membri

     call derxx( us, d2us, Nt )
     us( 1 : Nt - 1 ) = us( 1 : Nt - 1 ) + dt * nu * d2us( 1 : Nt - 1 )

! Condizioni al bordo

     alfa = 0.d0
     beta = 0.d0

     do i = 1, Nt - 1
        alfa = alfa + alp( i ) * us( i )
        beta = beta + alm( i ) * us( i )
     enddo
     alfa = -alfa
     beta = -beta
! Funzione nulla sul bordo (destro e sinistro) -> Dirichlet
!     us( Nt ) = ( alfa - beta ) / 2.d0
!     us( Ntp1 ) = ( alfa + beta ) / 2.d0
! Derivata nulla sul bordo (destro e sinistro) -> Neumann
     us( Nt ) = ( beta + alfa ) / ( 2.d0 * alp( Nt ) )
     us( Ntp1 ) = ( alfa - beta ) / ( 2.d0 * alp( Ntp1 ) )
     us( Ntp2 ) = 0.d0

! Stampa variabili ad ogni tempo di stampa

     if ( mod( istep, Nprint ) == 0 ) then
        call cheb( uf, us, wch, chfns, Nt, -1 )
        tt = istep * dt
        call output( xch, uf, tt, Ntp1 )
        open ( file='integrale.dat', unit=16, status = 'UNKNOWN', access = 'APPEND' )
        write(16,*) tt, trapezi( uf, xch, Ntp1 )
        close( unit = 16 )
        call derx( us, d1us, Nt )
        call cheb( d1uf, d1us, wch, chfns, Nt, -1 )
        open ( file='bc.dat', unit=15, status = 'UNKNOWN', access = 'APPEND' )
        write( 15, * ) d1uf( 1 ), d1uf( Ntp1 )
        close( unit = 15 )
     endif

! Fine ciclo temporale

     enddo

contains

  subroutine output( x, f, t, n )

    double precision, intent(IN), dimension (:) :: x, f
    double precision, intent(IN) :: t
    integer, intent(IN) :: n
    integer :: i

    print *,'set title "t = ',t,'"'
    print *,'set xlabel "x"'
    print *,'set ylabel "f"'
!    print *,'set yrange [0:1]'
    print *,'plot "-" u 1:2 notitle w l'
    do i = 1, n
       print *, x(i), f(i)
    enddo
    print *,'e'

    return

  end subroutine output

  double precision function trapezi( f, xch, Ntp1 )

    integer :: Ntp1, i
    double precision, dimension(:):: f, xch
    double precision :: sum

    sum = 0.d0
    do i = 2, Ntp1
       sum = sum + 0.5d0 * ( xch( i ) - xch( i - 1 ) ) * &
            ( f( i ) + f( i - 1 ) )
    enddo
    trapezi = sum
    return

  end function trapezi

end program parabolica
